/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.collections.bytearray;

import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.builtu.collections.bytearray.ByteArray;
import org.builtu.exceptions.FatalError;
import org.junit.Test;


public class TestByteArraySpecific extends TestByteArray {
  
  /*@Test public void testRandom_TesteeAgainstRef() 
  {
    setSeed(1220270351432L);
    super.testRandom_TesteeAgainstRef();
  }*/
  
  @Test public void test_byte_add() {
    final ByteArray b = new ByteArray();
    b.byte_add((byte)100);
    Assert.assertEquals(1, b.size());
    Assert.assertEquals(100, b.byte_get(0));
  }

  @Test public void test_byte_add2() {
    final ByteArray b = new ByteArray();
    b.byte_add(0, (byte)100);
    Assert.assertEquals(1, b.size());
    Assert.assertEquals(100, b.byte_get(0));
    b.byte_add(0, (byte)102);
    Assert.assertEquals(2, b.size());
    Assert.assertEquals(102, b.byte_get(0));
    Assert.assertEquals(100, b.byte_get(1));
  }

  @Test public void test_byte_add3() {
    final byte [] toadd = new byte[] {1, 2};
    final ByteArray b = new ByteArray();
    b.byte_add(toadd);
    Assert.assertEquals(2, b.size());
    Assert.assertEquals(1, b.byte_get(0));
    Assert.assertEquals(2, b.byte_get(1));
  }

  @Test public void test_byte_add4() {
    final byte [] toadd = new byte[] {0, 1, 2, 3};
    final ByteArray b = new ByteArray();
    b.byte_add(0, toadd, 1, 3);
    Assert.assertEquals(2, b.size());
    Assert.assertEquals(1, b.byte_get(0));
    Assert.assertEquals(2, b.byte_get(1));
  }
  
  @Test public void test_byte_remove() {
    final ByteArray b = new ByteArray();
    b.add((byte)1);
    Assert.assertTrue(b.byte_remove((byte)1));
    Assert.assertEquals(0, b.size());
    b.add((byte)1);
    b.add((byte)2);
    b.add((byte)2);
    Assert.assertTrue(b.byte_remove((byte)2));
    Assert.assertEquals(2, b.size());
    Assert.assertEquals(1, b.byte_get(0));
    Assert.assertEquals(2, b.byte_get(1));
  }
  
  @Test public void test_byte_removeAll() {
    final ByteArray b = new ByteArray();
    b.add((byte)1);
    b.add((byte)2);
    b.add((byte)2);
    Assert.assertTrue(b.byte_removeAll(new byte[] {(byte)2}));
    Assert.assertEquals(1, b.size());
    Assert.assertEquals(1, b.byte_get(0));
  }

  @Override protected ListOperation<Byte> makeModifier(final List<Byte> ref)
  {
    if (rnd(2)==0) return super.makeModifier(ref);
    final Byte e = makeElement();
    final byte [] random_arr = new byte[rnd(ref.size())];
    final List<Byte> random_coll = new ArrayList<Byte>();
    for(int i=0; i<random_arr.length; i++) {
      random_arr[i] = (byte) rnd();
      random_coll.add(random_arr[i]);
    }
    final int idx = rnd(ref.size());
    final List<Byte> existing_coll = randomCollectionOfExistingElements(ref);
    final byte [] existing_arr = new byte[existing_coll.size()];
    for (int i=0; i<existing_arr.length; i++) 
        existing_arr[i] = existing_coll.get(i);
    

    final int opCode = rnd(6);
    switch (opCode) {
    case 0: return new ListOperation<Byte>(){
      @Override public Byte modifyList(final List<Byte> list)
      {
        if (list instanceof ByteArray)
          ((ByteArray) list).byte_add(e);
        else
          list.add(e);
        return null;
      }}; 
    case 1: return new ListOperation<Byte>(){
      @Override public Byte modifyList(final List<Byte> list)
      {
        if (list instanceof ByteArray) {
          return b2e(((ByteArray) list).byte_add(random_arr), e);
        }
        else
          return b2e(list.addAll(random_coll), e);
      }}; 
    case 2: return new ListOperation<Byte>(){
      @Override public Byte modifyList(final List<Byte> list)
      {
        if (list instanceof ByteArray) {
          return b2e(((ByteArray) list)
              .byte_add(idx, random_arr, 0, random_arr.length), e);
        }
        else
          return b2e(list.addAll(idx, random_coll), e);
      }}; 
    case 3: return new ListOperation<Byte>(){
      @Override public Byte modifyList(final List<Byte> list)
      {
        if (list instanceof ByteArray) {
          return b2e(((ByteArray) list).byte_remove(e), e);
        }
        else
          return b2e(list.remove(e), e);
      }}; 
    case 4: return new ListOperation<Byte>(){
      @Override public Byte modifyList(final List<Byte> list)
      {
        if (list instanceof ByteArray) {
          return b2e(((ByteArray) list).byte_removeAll(random_arr), e);
        }
        else
          return b2e(list.removeAll(random_coll), e);
      }}; 
    case 5: return new ListOperation<Byte>(){
      @Override public Byte modifyList(final List<Byte> list)
      {
        if (list instanceof ByteArray) {
          return b2e(((ByteArray) list).byte_removeAll(existing_arr), e);
        }
        else
          return b2e(list.removeAll(existing_coll), e);
      }}; 
      
    default: fail("Bad opcode "+opCode);
    }
    throw new FatalError("Can't reach here!!!");
  }

  
}
