/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.config.spring;

import org.builtu.config.module.ConfigUtil;
import org.builtu.config.module.Configurable;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.config.BeanPostProcessor;


public class SpringConfig implements BeanPostProcessor, BeanNameAware {
	//private static final Logger logger = Logger.getLogger(BeanFactoryPostProcessor.class);
	
	// beanName->property->value
	//private final Map<String, Map<String, String>> properties = new HashMap<String, Map<String, String>>();
	
	private String moduleName, beanName;
	
	public SpringConfig() {
	}
	
	public SpringConfig(final String moduleName) {
		setModuleName(moduleName);
	}
	
	public void setModuleName(final String moduleName) {
		this.moduleName = moduleName;
	}

	@Override public void setBeanName(final String beanName) {
		this.beanName = beanName;
	}
	
	@Override public Object postProcessAfterInitialization(final Object bean, final String name) 
		throws BeansException 
	{
		if (!(bean instanceof Configurable)) return bean;
		final Configurable configurable = (Configurable) bean;
		ConfigUtil.configure(moduleName(), configurable);
		return bean;
	}

	@Override public Object postProcessBeforeInitialization(final Object bean, final String name) 
		throws BeansException 
	{
		return bean;
	}

	private String moduleName () {
		final String res = moduleName != null ? moduleName : beanName;
		assert res != null : "Neither moduleName nor beanName was provided for this SptingConfig";
		return res;
	}

}
