/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.clown;

import java.util.ArrayList;

import junit.framework.Assert;

import org.builtu.testing.BasicTestCase;
import org.builtu.util.clown.FastClown;
import org.builtu.util.clown.JDKClown;
import org.junit.Test;


public class TestFastClownSpeed extends BasicTestCase {
	
	@Test public void testFastClownIsFasterThanJDK () {
		final ArrayList<ComplexClassBase> classes = new ArrayList<ComplexClassBase>();
		for (int i=0; i<100; i++) {
			classes.add(new ComplexClassSerializable(super.random));
		}
		
		final JDKClown jdk = new JDKClown();
		final long jdkStart = System.currentTimeMillis();
		jdk.deepClone(classes);
		final long jdkTime = System.currentTimeMillis()-jdkStart;
		
		final FastClown fast = new FastClown();
		final long fastStart = System.currentTimeMillis();
		fast.deepClone(classes);
		final long fastTime = System.currentTimeMillis()-fastStart;

		float speedup = ((float) jdkTime) / fastTime;
		
		System.out.println("FastClown is "+speedup+" times faster than JDKClown");
		
		Assert.assertTrue(fastTime <= jdkTime);
	}


}
