/**
	This file is part of BUILTU.

    BUILTU is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BUILTU is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser Public License for more details.

    You should have received a copy of the GNU Lesser Public License
    along with BUILTU.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2005-2010 Pasko Boris
 */
package org.builtu.util.mail;

import java.io.UnsupportedEncodingException;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Random;

import javax.mail.MessagingException;
import javax.mail.internet.MimeUtility;

import org.builtu.exceptions.FatalError;




public class MailUtil {
    
	private static final Random rnd = new Random();
	
    private static final String MESSAGE_TPL = 
    	"Message-ID: <{0}.{1}@yellteam.ru>\n"+
    	"Date: {2}\n"+
    	"From: {3}\n"+
    	"User-Agent: Thunderbird 2.0.0.23 (X11/20090817)\n"+
    	"MIME-Version: 1.0\n"+
    	"{4}"+
    	"Subject: {5}\n"+
    	"Content-Type: text/plain; charset=UTF-8; format=flowed\n"+
    	"Content-Transfer-Encoding: 8bit\n\n{6}";

    public static String makeMessage(
            final String from, 
            final String to,
            final String subject,
            final String body
    ) throws MessagingException {
    	final String[] tos = to.split(",");
    	return makeMessage(from, tos, subject, body);
    }
    
    private static String makeMessage(
            final String from, 
            final String[] to,
            final String subject,
            final String body
    ) throws MessagingException {
    	
    	final Date now = new Date();
    	
    	final Object [] args = {
    			makeHex(now), makeId(),
    			makeDate(now), 
    			from,
    			makeTo(to),
    			encodeHeader(subject),
    			body
    	};
    	
    	return MessageFormat.format(MESSAGE_TPL, args);
    }

	private static String encodeHeader(String subject) {
		try {
			return MimeUtility.encodeText(subject, "UTF-8", null);
		} catch (final UnsupportedEncodingException e) {
			throw new FatalError(e);
		}
	}

	private static String makeHex(final Date now) {
		final StringBuilder res = new StringBuilder();
		res.append(Integer.toHexString((int)(now.getTime()/1000)).toUpperCase());
		while (res.length()<8) res.insert(0, '0');
		return res.toString();
	}

	private static String makeId() {
		return String.valueOf(rnd.nextInt());
	}
	
	// Something like Mon, 21 Sep 2009 13:21:19 +0400
	private final static SimpleDateFormat unix = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", Locale.ENGLISH);

	private static String makeDate(final Date now) {
		return unix.format(now);
	}

	private static String makeTo(final String[] to) {
		final StringBuilder b = new StringBuilder(to.length << 6);
		for (final String t : to) {
			b.append("To: ").append(t.trim()).append("\n"); 
		}
		return b.toString();
	}

	
}
